/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* bprint.c - a printing utility for debugging output */
#include <string.h>
#include "lber-int.h"

#ifdef LDAP_DEBUG
/*
 * Print arbitrary stuff, for debugging.
 */

#  define BPLEN 48

void lber_bprint(char* data, int len) {
  static char hexdig[] = "0123456789abcdef";
  char out[BPLEN];
  int i = 0;

  memset(out, 0, BPLEN);
  for (;;) {
    if (len < 1) {
      char msg[BPLEN + 80];
      sprintf(msg, "\t%s\n", (i == 0) ? "(end)" : out);
      ber_err_print(msg);
      break;
    }

#  ifndef HEX
    if (isgraph((unsigned char)*data)) {
      out[i] = ' ';
      out[i + 1] = *data;
    } else {
#  endif
      out[i] = hexdig[(*data & 0xf0) >> 4];
      out[i + 1] = hexdig[*data & 0x0f];
#  ifndef HEX
    }
#  endif
    i += 2;
    len--;
    data++;

    if (i > BPLEN - 2) {
      char msg[BPLEN + 80];
      sprintf(msg, "\t%s\n", out);
      ber_err_print(msg);
      memset(out, 0, BPLEN);
      i = 0;
      continue;
    }
    out[i++] = ' ';
  }
}

#endif

void ber_err_print(char* data) {
#ifdef USE_DEBUG_WIN
  OutputDebugString(data);
#else
  fputs(data, stderr);
  fflush(stderr);
#endif
}
